import {
  CMType,
  EmbeddedComponent,
  type IVMS101,
  type OwnershipProof,
  type TransactionResponse,
  type V1Transaction,
} from '@notabene/javascript-sdk';
import { NotabeneSdk } from './notabene-sdk';

export interface TransactionResponseRN<V> extends TransactionResponse<V> {
  value: V;
  ivms101: IVMS101;
  proof?: OwnershipProof;
  txCreate?: V1Transaction;
  txUpdate?: V1Transaction;
  deepLink: string;
}

interface ParsedResponse<V> {
  type?: CMType;
  description?: string;
  message?: string;
  response?: TransactionResponseRN<V>;
}

function isTransactionResponse<V>(
  response: ParsedResponse<V>
): response is ParsedResponse<V> & { type: CMType.COMPLETE } {
  return response.type === CMType.COMPLETE;
}

function isCancelResponse<V>(
  response: ParsedResponse<V>
): response is ParsedResponse<V> & { type: CMType.CANCEL } {
  return response.type === CMType.CANCEL;
}

function isErrorResponse<V>(
  response: ParsedResponse<V>
): response is ParsedResponse<V> & { type: CMType.ERROR } {
  return response.type === CMType.ERROR;
}

export class EmbeddedComponentRN<V, O> extends EmbeddedComponent<V, O> {
  deepLink: string;
  constructor(url: string, deepLink: string, value: Partial<V>) {
    super(url, value);
    this.deepLink = deepLink;
  }

  async openModal(): Promise<TransactionResponseRN<V>> {
    try {
      // get the string response from Native
      const response = await openComponent(this.url, this.deepLink);

      // Parse the response to check for conclusive errors from the JS SDK
      const [_, fragment] = response.split('#');
      const params = new URLSearchParams(fragment);
      const encodedMessage = params.has('message')
        ? params.get('message')
        : undefined;

      const parsedResponse: ParsedResponse<V> = {
        ...(encodedMessage ? JSON.parse(encodedMessage) : {}),
      };

      if (isTransactionResponse<V>(parsedResponse)) {
        if (parsedResponse.response) {
          return parsedResponse.response;
        }
      }

      if (isCancelResponse<V>(parsedResponse)) {
        return Promise.reject({
          ...parsedResponse,
          message: 'User cancellation',
          deepLink: response,
        });
      }

      if (isErrorResponse<V>(parsedResponse)) {
        return Promise.reject({
          ...parsedResponse,
          message: parsedResponse?.description || parsedResponse?.message,
          deepLink: response,
        });
      }

      return Promise.reject({
        ...parsedResponse,
        message: 'Unknown error',
        deepLink: response,
      });
    } catch (ex) {
      const error = ex as Error;
      if (error.message?.includes('The operation couldn’t be completed')) {
        return Promise.reject('User cancellation');
      }
      return Promise.reject(error);
    }
  }
}

async function openComponent(url: string, deepLink: string): Promise<string> {
  return await NotabeneSdk.open(url, deepLink);
}
