import 'react-native-url-polyfill/auto';
import Notabene, {
  type CallbackOptions,
  type ConnectionOptions,
  type ConnectionRequest,
  type Deposit,
  type DepositRequest,
  type DepositRequestOptions,
  type NotabeneConfig,
  type TransactionOptions,
  type Withdrawal,
} from '@notabene/javascript-sdk';
import { EmbeddedComponentRN } from './embedded-component';

export {
  type TransactionResponseRN,
  EmbeddedComponentRN,
} from './embedded-component';

export interface NotabeneRNConfig extends NotabeneConfig {
  componentDeepLink?: string; // The deep link to the component within the app
  resultDeepLink: string; // The deep link to receive the result within the app
}

export default class NotabeneRN extends Notabene {
  componentDeepLink?: string;
  resultDeepLink: string;

  constructor(config: NotabeneRNConfig) {
    if (!config.uxUrl?.length) {
      throw new Error('Notabene `uxUrl` is required');
    }
    if (!config.nodeUrl?.length) {
      throw new Error('Notabene `nodeUrl` is required');
    }
    if (!config.authToken?.length) {
      throw new Error('Notabene `authToken` is required');
    }
    if (!config.resultDeepLink?.length) {
      throw new Error('Notabene `resultDeepLink` is required');
    }

    super(config);

    this.resultDeepLink = config.resultDeepLink;
    this.componentDeepLink = config.componentDeepLink;
  }

  /**
   * Creates a new embedded component
   *
   * @param path - The path of the component
   * @param value - Transaction data
   * @param options - Optional transaction options
   * @param callbacks - Optional callback configuration
   * @returns A new EmbeddedComponent instance
   * @internal
   */
  createComponent<V, O>(
    path: string,
    value: Partial<V>,
    options?: O,
    callbacks?: CallbackOptions
  ) {
    return new EmbeddedComponentRN<V, O>(
      this.componentUrl(path, value, options, callbacks),
      this.resultDeepLink,
      value
    );
  }

  /**
   * Generates a URL for a Notabene component
   *
   * @param path - The path of the component
   * @param value - Transaction data
   * @param configuration - Optional transaction configuration
   * @param callbacks - Optional callback configuration
   * @returns component URL
   * @internal
   */
  componentUrl<V, O>(
    path: string,
    value: V,
    configuration?: O,
    callbacks?: CallbackOptions
  ): string {
    const url = super.componentUrl(path, value, configuration, callbacks);
    const sdkUrl = new URL(url);
    sdkUrl.searchParams.set('redirect_uri', this.resultDeepLink);
    return sdkUrl.toString();
  }

  /**
   * Creates a connect component
   *
   * @param value - Connection request data
   * @param options - Optional transaction options
   * @param callbacks - Optional callback configuration
   * @returns A new EmbeddedComponent instance for connection
   */
  override createConnectWallet(
    value: ConnectionRequest,
    options?: ConnectionOptions,
    callbacks?: CallbackOptions
  ) {
    return this.createComponent<ConnectionRequest, ConnectionOptions>(
      'connect',
      value,
      options,
      callbacks
    );
  }

  /**
   * Creates a deposit request component
   *
   * @param value - Deposit request data
   * @param options - Optional transaction options
   * @param callbacks - Optional callback configuration
   * @returns A new EmbeddedComponent instance for deposit requests
   * @public
   */
  public createDepositRequest(
    value: DepositRequest,
    options?: DepositRequestOptions,
    callbacks?: CallbackOptions
  ) {
    return this.createComponent<DepositRequest, DepositRequestOptions>(
      'deposit-request',
      value,
      options,
      callbacks
    );
  }

  /**
   * Creates a withdrawal assist component
   *
   * @param value - Withdrawal transaction data
   * @param options - Optional transaction options
   * @param callbacks - Optional callback configuration
   * @returns A new EmbeddedComponent instance for withdrawal assistance
   */
  override createWithdrawalAssist(
    value: Partial<Withdrawal>,
    options?: TransactionOptions,
    callbacks?: CallbackOptions
  ) {
    return this.createComponent<Withdrawal, TransactionOptions>(
      'withdrawal-assist',
      value,
      options,
      callbacks
    );
  }

  /**
   * Creates a deposit assist component
   *
   * @param value - Deposit transaction data
   * @param options - Optional transaction options
   * @param callbacks - Optional callback configuration
   * @returns A new EmbeddedComponent instance for deposit assistance
   */
  override createDepositAssist(
    value: Partial<Deposit>,
    options?: TransactionOptions,
    callbacks?: CallbackOptions
  ) {
    return this.createComponent<Deposit, TransactionOptions>(
      'deposit-assist',
      value,
      options,
      callbacks
    );
  }
}
